@@ -206,7 +206,11 @@ Process(JSContext *cx, JSObject *obj, char *filename)
     }
     JS_SetThreadStackLimit(cx, stackLimit);
 
+#if defined(OSSP) && defined(XP_WIN)
+    if (filename) {
+#else
     if (!isatty(fileno(file))) {
+#endif
         /*
          * It's not interactive - just execute it.
          *
@@ -2492,6 +2496,11 @@ main(int argc, char **argv, char **envp)
     setbuf(stderr,0);
 #endif
 
+#if defined(OSSP) && defined(XP_WIN)
+    setvbuf(stderr, 0, _IONBF, 0);
+    setvbuf(stdout, 0, _IONBF, 0);
+#endif
+
     gErrFile = stderr;
     gOutFile = stdout;
 
