@@ -52,6 +52,11 @@
 /* ----------------- Platform-specific includes and defines ----------------- */
 #if defined(XP_WIN) || defined(XP_OS2)
 #   include <direct.h>
+#ifdef OSSP
+#   include <dirent.h>
+#   include <fcntl.h>
+#   include <time.h>
+#endif
 #   include <io.h>
 #   include <sys/types.h>
 #   include <sys/stat.h>
@@ -60,6 +65,10 @@
 #   define CURRENT_DIR          "c:\\"
 #   define POPEN                _popen
 #   define PCLOSE               _pclose
+#ifdef OSSP
+#   undef mkdir
+#   define mkdir(file, mode)    _mkdir(file)
+#endif
 #elif defined(XP_UNIX) || defined(XP_BEOS)
 #   include <strings.h>
 #   include <stdio.h>
